window.Gleam.initSearch([{"doc":"eensy","title":"eensy","content":"# eensy\n\nVery early alpha stage. Please just use it for hobby projects. Open to contributions.\n\n[![Package Version](https://img.shields.io/hexpm/v/eensy)](https://hex.pm/packages/eensy)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/eensy/)\n\n```sh\ngleam add eensy\n```\n```gleam\nimport eensy.{get_system_platform}\nimport eensy/gpio\nimport eensy/otp/task\nimport gleam/erlang/process\nimport gleam/result.{try}\n\npub fn start() -&gt; Nil {\n  case eensy.get_system_platform() {\n    eensy.Esp32 -&gt; {\n      // Set pin mode to output\n      let _ = gpio.set_pin_mode(pin(), gpio.Output)\n\n      // Start blinking\n      let _blinky = process.start(fn() { loop(pin(), gpio.High) }, False)\n\n      Nil\n    }\n    _ -&gt; Nil\n  }\n\n  process.sleep_forever()\n}\n\nfn pin() {\n  case get_system_platform() {\n    eensy.Esp32 | eensy.Pico -&gt; 2\n    _ -&gt; 0\n  }\n}\n\nfn loop(pin, level) {\n  use _ &lt;- try(gpio.digital_write(pin, toggle(level)))\n  process.sleep(case level {\n    gpio.High -&gt; 150\n    gpio.Low -&gt; 300\n  })\n  loop(pin, toggle(level))\n}\n\nfn toggle(level) {\n  case level {\n    gpio.High -&gt; gpio.Low\n    _ -&gt; gpio.High\n  }\n}\n\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/eensy&gt;.\n\n## Development\n\n```sh\ngleam run -m eensy_dev_tools pack # Run devtools\natomvm ./build/.atomvm/&lt;project_name&gt;.release.avm # Run the avm file in a unix atomvm instance\ngleam test  # Run the tests\n```\n\n","url":"index.html"},{"doc":"eensy","title":"eensy","content":"","url":"#"},{"doc":"eensy","title":"eensy","content":"# 02 The GPIO Guide\n\nThe idea of Eensy gpio module is to provide type-safe functions for properly handle input/output from the microcontroller dev kit hardware.\n\n## A quick example\n\nIn the next initial example related the gpio module. We are assuming a setup where we have 7 \ninput buttons distributed in the pins `[12, 14, 27, 26, 25, 32, 33]` and we&#39;re going to flash the main LED PIN when any of them are pressed. \n\n```gleam\nimport eensy.{get_system_platform}\nimport eensy/gpio\nimport gleam/erlang/process\nimport gleam/io\nimport gleam/list\n\npub fn start() -&gt; Nil {\n  case eensy.get_system_platform() {\n    eensy.Esp32 -&gt; {\n      let _ = gpio.set_pin_mode(internal_blink_pin(), gpio.Output)\n\n      let _ =\n        process.start(\n          fn() {\n            [12, 14, 27, 26, 25, 32, 33]\n            |&gt; init_inputs\n            |&gt; check_inputs\n          },\n          False,\n        )\n\n      Nil\n    }\n    _ -&gt; Nil\n  }\n\n  process.sleep_forever()\n}\n\nfn init_inputs(pins) {\n  list.map(pins, fn(pin) {\n    process.sleep(10)\n    case gpio.set_pin_pull(pin, gpio.Up) {\n      Error(_) -&gt; io.println(&quot;set_pin_pull error&quot;)\n      _ -&gt; Nil\n    }\n    process.sleep(10)\n    case gpio.set_pin_mode(pin, gpio.Input) {\n      Error(_) -&gt; io.println(&quot;init_inputs error&quot;)\n      _ -&gt; Nil\n    }\n  })\n  pins\n}\n\nfn check_inputs(pins) {\n  list.map(pins, fn(pin) {\n    process.sleep(10)\n    case gpio.digital_read(pin) {\n      Ok(gpio.Low) -&gt; {\n        let _ = gpio.digital_write(internal_blink_pin(), gpio.High)\n        Nil\n      }\n      Ok(gpio.High) -&gt; {\n        let _ = gpio.digital_write(internal_blink_pin(), gpio.Low)\n        Nil\n      }\n      _ -&gt; io.println(&quot;error&quot;)\n    }\n  })\n  process.sleep(20)\n  check_inputs(pins)\n  pins\n}\n\nfn internal_blink_pin() {\n  case get_system_platform() {\n    eensy.Esp32 | eensy.Pico -&gt; 2\n    _ -&gt; 0\n  }\n}\n```\n\n","url":"guide/02-gpio-guide.html"},{"doc":"eensy","title":"PlatformName","content":"pub type PlatformName {\n  GenericUnix\n  Emscripten\n  Esp32\n  Pico\n  Stm32\n}\n\nGenericUnix\n\n\nEmscripten\n\n\nEsp32\n\n\nPico\n\n\nStm32\n\n\nSynonyms:\neensy.PlatformName\neensy PlatformName","url":"eensy.html#PlatformName"},{"doc":"eensy","title":"erlang_display","content":"pub fn erlang_display(any: a) -&gt; Nil\n using erlang:display directly from gleam for debug purposes\nSynonyms:\neensy.erlang_display\neensy erlang_display","url":"eensy.html#erlang_display"},{"doc":"eensy","title":"erlang_garbage_collect","content":"pub fn erlang_garbage_collect() -&gt; Nil\n using erlang:garbage_collect directly from gleam for forcing vm behaviour\nSynonyms:\neensy.erlang_garbage_collect\neensy erlang_garbage_collect","url":"eensy.html#erlang_garbage_collect"},{"doc":"eensy","title":"get_system_platform","content":"pub fn get_system_platform() -&gt; PlatformName\n get the running platform name\nSynonyms:\neensy.get_system_platform\neensy get_system_platform","url":"eensy.html#get_system_platform"},{"doc":"eensy","title":"eensy","content":"","url":"eensy.html"},{"doc":"eensy/gpio","title":"Direction","content":"pub type Direction {\n  Input\n  Output\n  OutputOd\n}\n\nInput\n\n\nOutput\n\n\nOutputOd\n\n\nSynonyms:\neensy/gpio.Direction\neensy/gpio Direction","url":"eensy/gpio.html#Direction"},{"doc":"eensy/gpio","title":"Level","content":"pub type Level {\n  Low\n  High\n}\n\nLow\n\n\nHigh\n\n\nSynonyms:\neensy/gpio.Level\neensy/gpio Level","url":"eensy/gpio.html#Level"},{"doc":"eensy/gpio","title":"Pull","content":"pub type Pull {\n  Up\n  Down\n}\n\nUp\n\n\nDown\n\n\nSynonyms:\neensy/gpio.Pull\neensy/gpio Pull","url":"eensy/gpio.html#Pull"},{"doc":"eensy/gpio","title":"digital_read","content":"pub fn digital_read(pin: Int) -&gt; Result(Level, Nil)\n Read from gpio pin\nSynonyms:\neensy/gpio.digital_read\neensy/gpio digital_read","url":"eensy/gpio.html#digital_read"},{"doc":"eensy/gpio","title":"digital_write","content":"pub fn digital_write(pin: Int, level: Level) -&gt; Result(Int, Nil)\n Write to gpio pin\nSynonyms:\neensy/gpio.digital_write\neensy/gpio digital_write","url":"eensy/gpio.html#digital_write"},{"doc":"eensy/gpio","title":"init","content":"pub fn init(pin: Int) -&gt; Nil\n Init pin\nSynonyms:\neensy/gpio.init\neensy/gpio init","url":"eensy/gpio.html#init"},{"doc":"eensy/gpio","title":"set_pin_mode","content":"pub fn set_pin_mode(\n  pin: Int,\n  direction: Direction,\n) -&gt; Result(Int, Nil)\n Set gpio pin direction\nSynonyms:\neensy/gpio.set_pin_mode\neensy/gpio set_pin_mode","url":"eensy/gpio.html#set_pin_mode"},{"doc":"eensy/gpio","title":"set_pin_pull","content":"pub fn set_pin_pull(pin: Int, pull: Pull) -&gt; Result(Nil, Nil)\n Set pin pull\nSynonyms:\neensy/gpio.set_pin_pull\neensy/gpio set_pin_pull","url":"eensy/gpio.html#set_pin_pull"},{"doc":"eensy/gpio","title":"start","content":"pub fn start() -&gt; Result(Int, Nil)\n Start gpio\nSynonyms:\neensy/gpio.start\neensy/gpio start","url":"eensy/gpio.html#start"},{"doc":"eensy/gpio","title":"eensy/gpio","content":"","url":"eensy/gpio.html"},{"doc":"eensy/http/server","title":"eensy/http/server","content":"","url":"eensy/http/server.html"},{"doc":"eensy/i2c","title":"OpenParameters","content":"pub type OpenParameters {\n  Scl(Int)\n  Sda(Int)\n  ClockSpeedHz(Int)\n}\n\nScl(Int)\n\n\nSda(Int)\n\n\nClockSpeedHz(Int)\n\n\nSynonyms:\neensy/i2c.OpenParameters\neensy/i2c OpenParameters","url":"eensy/i2c.html#OpenParameters"},{"doc":"eensy/i2c","title":"open","content":"pub fn open(parameters: List(OpenParameters)) -&gt; Int\n\nSynonyms:\neensy/i2c.open\neensy/i2c open","url":"eensy/i2c.html#open"},{"doc":"eensy/i2c","title":"eensy/i2c","content":"","url":"eensy/i2c.html"},{"doc":"eensy/network","title":"start","content":"pub fn start() -&gt; Result(Nil, Nil)\n Start Access Point in unsecure mode\nSynonyms:\neensy/network.start\neensy/network start","url":"eensy/network.html#start"},{"doc":"eensy/network","title":"eensy/network","content":"","url":"eensy/network.html"},{"doc":"eensy/otp/system","title":"DebugOption","content":"pub type DebugOption {\n  NoDebug\n}\n\nNoDebug\n\n\nSynonyms:\neensy/otp/system.DebugOption\neensy/otp/system DebugOption","url":"eensy/otp/system.html#DebugOption"},{"doc":"eensy/otp/system","title":"DebugState","content":"pub type DebugState\n\n\nSynonyms:\neensy/otp/system.DebugState\neensy/otp/system DebugState","url":"eensy/otp/system.html#DebugState"},{"doc":"eensy/otp/system","title":"Mode","content":"pub type Mode {\n  Running\n  Suspended\n}\n\nRunning\n Currently handling message as normal.\n\nSuspended\n Termporarily not handling messages, other than system messages.\n\nSynonyms:\neensy/otp/system.Mode\neensy/otp/system Mode","url":"eensy/otp/system.html#Mode"},{"doc":"eensy/otp/system","title":"StatusInfo","content":"pub type StatusInfo {\n  StatusInfo(\n    module: Atom,\n    parent: Pid,\n    mode: Mode,\n    debug_state: DebugState,\n    state: Dynamic,\n  )\n}\n\nStatusInfo(\n  module: Atom,\n  parent: Pid,\n  mode: Mode,\n  debug_state: DebugState,\n  state: Dynamic,\n)\n\n\nSynonyms:\neensy/otp/system.StatusInfo\neensy/otp/system StatusInfo","url":"eensy/otp/system.html#StatusInfo"},{"doc":"eensy/otp/system","title":"SystemMessage","content":"pub type SystemMessage {\n  Resume(fn() -&gt; Nil)\n  Suspend(fn() -&gt; Nil)\n  GetState(fn(Dynamic) -&gt; Nil)\n  GetStatus(fn(StatusInfo) -&gt; Nil)\n}\n\nResume(fn() -&gt; Nil)\n\n\nSuspend(fn() -&gt; Nil)\n\n\nGetState(fn(Dynamic) -&gt; Nil)\n\n\nGetStatus(fn(StatusInfo) -&gt; Nil)\n\n\nSynonyms:\neensy/otp/system.SystemMessage\neensy/otp/system SystemMessage","url":"eensy/otp/system.html#SystemMessage"},{"doc":"eensy/otp/system","title":"debug_state","content":"pub fn debug_state(a: List(DebugOption)) -&gt; DebugState\n\nSynonyms:\neensy/otp/system.debug_state\neensy/otp/system debug_state","url":"eensy/otp/system.html#debug_state"},{"doc":"eensy/otp/system","title":"get_state","content":"pub fn get_state(from from: Pid) -&gt; Dynamic\n Get the state of a given OTP compatible process. This function is only\n intended for debugging.\n\n Requires Erlang/OTP 26.1 or newer, as the underlying interface changed\n in [OTP-18633][1] from a literal type to a result type.\n\n For more information see the [Erlang documentation][2].\n\n [1]: https://www.erlang.org/patches/otp-26.1#stdlib-5.1\n [2]: https://erlang.org/doc/man/sys.html#get_state-1\n\nSynonyms:\neensy/otp/system.get_state\neensy/otp/system get_state","url":"eensy/otp/system.html#get_state"},{"doc":"eensy/otp/system","title":"resume","content":"pub fn resume(pid: Pid) -&gt; Nil\n Request a suspended OTP compatible process to result, causing it to handle\n all messages rather than only system messages.\n\n For more information see the [Erlang documentation][1].\n\n [1]: https://erlang.org/doc/man/sys.html#resume-1\n\nSynonyms:\neensy/otp/system.resume\neensy/otp/system resume","url":"eensy/otp/system.html#resume"},{"doc":"eensy/otp/system","title":"suspend","content":"pub fn suspend(pid: Pid) -&gt; Nil\n Request an OTP compatible process to suspend, causing it to only handle\n system messages.\n\n For more information see the [Erlang documentation][1].\n\n [1]: https://erlang.org/doc/man/sys.html#suspend-1\n\nSynonyms:\neensy/otp/system.suspend\neensy/otp/system suspend","url":"eensy/otp/system.html#suspend"},{"doc":"eensy/otp/system","title":"eensy/otp/system","content":"","url":"eensy/otp/system.html"},{"doc":"eensy/otp/actor","title":"ErlangStartResult","content":"pub type ErlangStartResult =\n  Result(Pid, Dynamic)\n An Erlang supervisor compatible process start result.\n\n\nSynonyms:\neensy/otp/actor.ErlangStartResult\neensy/otp/actor ErlangStartResult","url":"eensy/otp/actor.html#ErlangStartResult"},{"doc":"eensy/otp/actor","title":"InitResult","content":"pub type InitResult(state, message) {\n  Ready(state: state, selector: Selector(message))\n  Failed(String)\n}\n The type used to indicate whether an actor has started successfully or not.\n\nReady(state: state, selector: Selector(message))\n The actor has successfully initialised. The actor can start handling\n messages and actor&#39;s channel sender can be returned to the parent\n process.\n\n\nFailed(String)\n The actor has failed to initialise. The actor shuts down and an error is\n returned to the parent process.\n\n\nSynonyms:\neensy/otp/actor.InitResult\neensy/otp/actor InitResult","url":"eensy/otp/actor.html#InitResult"},{"doc":"eensy/otp/actor","title":"Next","content":"pub type Next(message, state) {\n  Continue(state: state, selector: Option(Selector(message)))\n  Stop(ExitReason)\n}\n The type used to indicate what to do after handling a message.\n\nContinue(state: state, selector: Option(Selector(message)))\n Continue handling messages.\n\n An optional selector can be provided to changes the messages that the\n actor is handling. This replaces any selector that was previously given\n in the actor&#39;s `init` callback, or in any previous `Next` value.\n\n\nStop(ExitReason)\n Stop handling messages and shut down.\n\n\nSynonyms:\neensy/otp/actor.Next\neensy/otp/actor Next","url":"eensy/otp/actor.html#Next"},{"doc":"eensy/otp/actor","title":"Spec","content":"pub type Spec(state, msg) {\n  Spec(\n    init: fn() -&gt; InitResult(state, msg),\n    init_timeout: Int,\n    loop: fn(msg, state) -&gt; Next(msg, state),\n  )\n}\n This data structure holds all the values required by the `start_spec`\n function in order to create an actor.\n\n If you do not need to configure the initialisation behaviour of your actor\n consider using the `start` function.\n\nSpec(\n  init: fn() -&gt; InitResult(state, msg),\n  init_timeout: Int,\n  loop: fn(msg, state) -&gt; Next(msg, state),\n)\n\ninit\n&lt;p&gt;The initialisation functionality for the actor. This function is called\njust after the actor starts but before the channel sender is returned\nto the parent.&lt;/p&gt;\n&lt;p&gt;This function is used to ensure that any required data or state is\ncorrect. If this function returns an error it means that the actor has\nfailed to start and an error is returned to the parent.&lt;/p&gt;\n\ninit_timeout\n&lt;p&gt;How many milliseconds the &lt;code&gt;init&lt;/code&gt; function has to return before it is\nconsidered to have taken too long and failed.&lt;/p&gt;\n\nloop\n&lt;p&gt;This function is called to handle each message that the actor receives.&lt;/p&gt;\n\nSynonyms:\neensy/otp/actor.Spec\neensy/otp/actor Spec","url":"eensy/otp/actor.html#Spec"},{"doc":"eensy/otp/actor","title":"StartError","content":"pub type StartError {\n  InitTimeout\n  InitFailed(ExitReason)\n  InitCrashed(Dynamic)\n}\n\nInitTimeout\n\n\nInitFailed(ExitReason)\n\n\nInitCrashed(Dynamic)\n\n\nSynonyms:\neensy/otp/actor.StartError\neensy/otp/actor StartError","url":"eensy/otp/actor.html#StartError"},{"doc":"eensy/otp/actor","title":"StartResult","content":"pub type StartResult(msg) =\n  Result(Subject(msg), StartError)\n The result of starting a Gleam actor.\n\n This type is compatible with Gleam supervisors. If you wish to convert it\n to a type compatible with Erlang supervisors see the `ErlangStartResult`\n type and `erlang_start_result` function.\n\n\nSynonyms:\neensy/otp/actor.StartResult\neensy/otp/actor StartResult","url":"eensy/otp/actor.html#StartResult"},{"doc":"eensy/otp/actor","title":"call","content":"pub fn call(\n  subject: Subject(a),\n  make_message: fn(Subject(b)) -&gt; a,\n  timeout: Int,\n) -&gt; b\n Send a synchronous message and wait for a response from the receiving\n process.\n\n If a reply is not received within the given timeout then the sender process\n crashes. If you wish to receive a `Result` rather than crashing see the\n `process.try_call` function.\n\n This is a re-export of `process.call`, for the sake of convenience.\n\nSynonyms:\neensy/otp/actor.call\neensy/otp/actor call","url":"eensy/otp/actor.html#call"},{"doc":"eensy/otp/actor","title":"continue","content":"pub fn continue(state: a) -&gt; Next(b, a)\n\nSynonyms:\neensy/otp/actor.continue\neensy/otp/actor continue","url":"eensy/otp/actor.html#continue"},{"doc":"eensy/otp/actor","title":"send","content":"pub fn send(subject: Subject(a), msg: a) -&gt; Nil\n Send a message over a given channel.\n\n This is a re-export of `process.send`, for the sake of convenience.\n\nSynonyms:\neensy/otp/actor.send\neensy/otp/actor send","url":"eensy/otp/actor.html#send"},{"doc":"eensy/otp/actor","title":"start","content":"pub fn start(\n  state: a,\n  loop: fn(b, a) -&gt; Next(b, a),\n) -&gt; Result(Subject(b), StartError)\n Start an actor with a given initial state and message handling loop\n function.\n\n This function returns a `Result` but it will always be `Ok` so it is safe\n to use with `assert` if you are not starting this actor as part of a\n supervision tree.\n\n If you wish to configure the initialisation behaviour of a new actor see\n the `Spec` record and the `start_spec` function.\n\nSynonyms:\neensy/otp/actor.start\neensy/otp/actor start","url":"eensy/otp/actor.html#start"},{"doc":"eensy/otp/actor","title":"start_spec","content":"pub fn start_spec(\n  spec: Spec(a, b),\n) -&gt; Result(Subject(b), StartError)\n Start an actor from a given specification. If the actor&#39;s `init` function\n returns an error or does not return within `init_timeout` then an error is\n returned.\n\n If you do not need to specify the initialisation behaviour of your actor\n consider using the `start` function.\n\nSynonyms:\neensy/otp/actor.start_spec\neensy/otp/actor start_spec","url":"eensy/otp/actor.html#start_spec"},{"doc":"eensy/otp/actor","title":"to_erlang_start_result","content":"pub fn to_erlang_start_result(\n  res: Result(Subject(a), StartError),\n) -&gt; Result(Pid, Dynamic)\n Convert a Gleam actor start result into an Erlang supervisor-compatible\n process start result.\n\nSynonyms:\neensy/otp/actor.to_erlang_start_result\neensy/otp/actor to_erlang_start_result","url":"eensy/otp/actor.html#to_erlang_start_result"},{"doc":"eensy/otp/actor","title":"with_selector","content":"pub fn with_selector(\n  value: Next(a, b),\n  selector: Selector(a),\n) -&gt; Next(a, b)\n Provide a selector to change the messages that the actor is handling\n going forward. This replaces any selector that was previously given\n in the actor&#39;s `init` callback, or in any previous `Next` value.\n\nSynonyms:\neensy/otp/actor.with_selector\neensy/otp/actor with_selector","url":"eensy/otp/actor.html#with_selector"},{"doc":"eensy/otp/actor","title":"eensy/otp/actor","content":" This module provides the _Actor_ abstraction, one of the most common\n building blocks of Gleam OTP programs.\n \n An Actor is a process like any other BEAM process and can be be used to hold\n state, execute code, and communicate with other processes by sending and\n receiving messages. The advantage of using the actor abstraction over a bare\n process is that it provides a single interface for commonly needed\n functionality, including support for the [tracing and debugging\n features in OTP][erlang-sys].\n\n Gleam&#39;s Actor is similar to Erlang&#39;s `gen_server` and Elixir&#39;s `GenServer`\n but differs in that it offers a fully typed interface. This different API is\n why Gleam uses the name Actor rather than some variation of generic-server.\n\n [erlang-sys]: https://www.erlang.org/doc/man/sys.html\n\n ## Example\n\n An Actor can be used to create a client-server interaction between an Actor\n (the server) and other processes (the clients). In this example we have an\n Actor that works as a stack, allowing clients to push and pop elements.\n\n ```gleam\n pub fn main() {\n   // Start the actor with initial state of an empty list, and the\n   // `handle_message` callback function (defined below).\n   // We assert that it starts successfully.\n   // \n   // In real-world Gleam OTP programs we would likely write wrapper functions\n   // called `start`, `push` `pop`, `shutdown` to start and interact with the\n   // Actor. We are not doing that here for the sake of showing how the Actor \n   // API works.\n   let assert Ok(my_actor) = actor.start([], handle_message)\n \n   // We can send a message to the actor to push elements onto the stack.\n   process.send(my_actor, Push(&quot;Joe&quot;))\n   process.send(my_actor, Push(&quot;Mike&quot;))\n   process.send(my_actor, Push(&quot;Robert&quot;))\n \n   // The `Push` message expects no response, these messages are sent purely for\n   // the side effect of mutating the state held by the actor.\n   //\n   // We can also send the `Pop` message to take a value off of the actor&#39;s\n   // stack. This message expects a response, so we use `process.call` to send a\n   // message and wait until a reply is received.\n   //\n   // In this instance we are giving the actor 10 milliseconds to reply, if the\n   // `call` function doesn&#39;t get a reply within this time it will panic and\n   // crash the client process.\n   let assert Ok(&quot;Robert&quot;) = process.call(my_actor, Pop, 10)\n   let assert Ok(&quot;Mike&quot;) = process.call(my_actor, Pop, 10)\n   let assert Ok(&quot;Joe&quot;) = process.call(my_actor, Pop, 10)\n \n   // The stack is now empty, so if we pop again the actor replies with an error.\n   let assert Error(Nil) = process.call(my_actor, Pop, 10)\n \n   // Lastly, we can send a message to the actor asking it to shut down.\n   process.send(my_actor, Shutdown)\n }\n ```\n\n Here is the code that is used to implement this actor:\n\n ```gleam\n import gleam/erlang/process.{type Subject}\n import gleam/otp/actor\n\n // First step of implementing the stack Actor is to define the message type that\n // it can receive.\n //\n // The type of the elements in the stack is not fixed so a type parameter is used\n // for it instead of a concrete type such as `String` or `Int`.\n pub type Message(element) {\n   // The `Shutdown` message is used to tell the actor to stop.\n   // It is the simplest message type, it contains no data.\n   Shutdown\n \n   // The `Push` message is used to add a new element to the stack.\n   // It contains the item to add, the type of which is the `element`\n   // parameterised type.\n   Push(push: element)\n \n   // The `Pop` message is used to remove an element from the stack.\n   // It contains a `Subject`, which is used to send the response back to the\n   // message sender. In this case the reply is of type `Result(element, Nil)`.\n   Pop(reply_with: Subject(Result(element, Nil)))\n }\n \n // The last part is to implement the `handle_message` callback function.\n //\n // This function is called by the Actor for each message it receives.\n // Actor is single threaded and only does one thing at a time, so it handles\n // messages sequentially and one at a time, in the order they are received.\n //\n // The function takes the message and the current state, and returns a data\n // structure that indicates what to do next, along with the new state.\n fn handle_message(\n  message: Message(e),\n  stack: List(e),\n ) -&gt; actor.Next(Message(e), List(e)) {\n   case message {\n     // For the `Shutdown` message we return the `actor.Stop` value, which causes\n     // the actor to discard any remaining messages and stop.\n     Shutdown -&gt; actor.Stop(process.Normal)\n \n     // For the `Push` message we add the new element to the stack and return\n     // `actor.continue` with this new stack, causing the actor to process any\n     // queued messages or wait for more.\n     Push(value) -&gt; {\n       let new_state = [value, ..stack]\n       actor.continue(new_state)\n     }\n \n     // For the `Pop` message we attempt to remove an element from the stack,\n     // sending it or an error back to the caller, before continuing.\n     Pop(client) -&gt;\n       case stack {\n         [] -&gt; {\n           // When the stack is empty we can&#39;t pop an element, so we send an\n           // error back.\n           process.send(client, Error(Nil))\n           actor.continue([])\n         }\n \n         [first, ..rest] -&gt; {\n           // Otherwise we send the first element back and use the remaining\n           // elements as the new state.\n           process.send(client, Ok(first))\n           actor.continue(rest)\n         }\n       }\n   }\n }\n ```","url":"eensy/otp/actor.html"},{"doc":"eensy/otp/static_supervisor","title":"AutoShutdown","content":"pub type AutoShutdown {\n  Never\n  AnySignificant\n  AllSignificant\n}\n A supervisor can be configured to automatically shut itself down with exit\n reason shutdown when significant children terminate with the auto_shutdown\n key in the above map.\nNever\n Automic shutdown is disabled. This is the default setting.\n\n With auto_shutdown set to never, child specs with the significant flag\n set to true are considered invalid and will be rejected.\n\nAnySignificant\n The supervisor will shut itself down when any significant child\n terminates, that is, when a transient significant child terminates\n normally or when a temporary significant child terminates normally or\n abnormally.\n\nAllSignificant\n The supervisor will shut itself down when all significant children have\n terminated, that is, when the last active significant child terminates.\n The same rules as for any_significant apply.\n\nSynonyms:\neensy/otp/static_supervisor.AutoShutdown\neensy/otp/static_supervisor AutoShutdown","url":"eensy/otp/static_supervisor.html#AutoShutdown"},{"doc":"eensy/otp/static_supervisor","title":"Builder","content":"pub opaque type Builder\n\n\nSynonyms:\neensy/otp/static_supervisor.Builder\neensy/otp/static_supervisor Builder","url":"eensy/otp/static_supervisor.html#Builder"},{"doc":"eensy/otp/static_supervisor","title":"ChildBuilder","content":"pub opaque type ChildBuilder\n\n\nSynonyms:\neensy/otp/static_supervisor.ChildBuilder\neensy/otp/static_supervisor ChildBuilder","url":"eensy/otp/static_supervisor.html#ChildBuilder"},{"doc":"eensy/otp/static_supervisor","title":"ChildType","content":"pub type ChildType {\n  Worker(shutdown_ms: Int)\n  Supervisor\n}\n\nWorker(shutdown_ms: Int)\n\nshutdown_ms\n&lt;p&gt;The number of milliseconds the child is given to shut down. The\nsupervisor tells the child process to terminate by calling\n&lt;code&gt;exit(Child,shutdown)&lt;/code&gt; and then wait for an exit signal with reason\nshutdown back from the child process. If no exit signal is received\nwithin the specified number of milliseconds, the child process is\nunconditionally terminated using &lt;code&gt;exit(Child,kill)&lt;/code&gt;.&lt;/p&gt;\n\nSupervisor\n\n\nSynonyms:\neensy/otp/static_supervisor.ChildType\neensy/otp/static_supervisor ChildType","url":"eensy/otp/static_supervisor.html#ChildType"},{"doc":"eensy/otp/static_supervisor","title":"Restart","content":"pub type Restart {\n  Permanent\n  Transient\n  Temporary\n}\n Restart defines when a terminated child process must be restarted. \nPermanent\n A permanent child process is always restarted.\n\nTransient\n A transient child process is restarted only if it terminates abnormally,\n that is, with another exit reason than `normal`, `shutdown`, or\n `{shutdown,Term}`.\n\nTemporary\n A temporary child process is never restarted (even when the supervisor&#39;s\n restart strategy is `RestForOne` or `OneForAll` and a sibling&#39;s death\n causes the temporary process to be terminated).\n\nSynonyms:\neensy/otp/static_supervisor.Restart\neensy/otp/static_supervisor Restart","url":"eensy/otp/static_supervisor.html#Restart"},{"doc":"eensy/otp/static_supervisor","title":"Strategy","content":"pub type Strategy {\n  OneForOne\n  OneForAll\n  RestForOne\n}\n\nOneForOne\n If one child process terminates and is to be restarted, only that child\n process is affected. This is the default restart strategy.\n\nOneForAll\n If one child process terminates and is to be restarted, all other child\n processes are terminated and then all child processes are restarted.\n\nRestForOne\n If one child process terminates and is to be restarted, the &#39;rest&#39; of the\n child processes (that is, the child processes after the terminated child\n process in the start order) are terminated. Then the terminated child\n process and all child processes after it are restarted.\n\nSynonyms:\neensy/otp/static_supervisor.Strategy\neensy/otp/static_supervisor Strategy","url":"eensy/otp/static_supervisor.html#Strategy"},{"doc":"eensy/otp/static_supervisor","title":"add","content":"pub fn add(builder: Builder, child: ChildBuilder) -&gt; Builder\n Add a child to the supervisor.\nSynonyms:\neensy/otp/static_supervisor.add\neensy/otp/static_supervisor add","url":"eensy/otp/static_supervisor.html#add"},{"doc":"eensy/otp/static_supervisor","title":"auto_shutdown","content":"pub fn auto_shutdown(\n  builder: Builder,\n  value: AutoShutdown,\n) -&gt; Builder\n A supervisor can be configured to automatically shut itself down with\n exit reason shutdown when significant children terminate.\nSynonyms:\neensy/otp/static_supervisor.auto_shutdown\neensy/otp/static_supervisor auto_shutdown","url":"eensy/otp/static_supervisor.html#auto_shutdown"},{"doc":"eensy/otp/static_supervisor","title":"new","content":"pub fn new(strategy strategy: Strategy) -&gt; Builder\n\nSynonyms:\neensy/otp/static_supervisor.new\neensy/otp/static_supervisor new","url":"eensy/otp/static_supervisor.html#new"},{"doc":"eensy/otp/static_supervisor","title":"restart","content":"pub fn restart(\n  child: ChildBuilder,\n  restart: Restart,\n) -&gt; ChildBuilder\n When the child is to be restarted. See the `Restart` documentation for\n more.\n\n The default value for restart is `Permenent`.\nSynonyms:\neensy/otp/static_supervisor.restart\neensy/otp/static_supervisor restart","url":"eensy/otp/static_supervisor.html#restart"},{"doc":"eensy/otp/static_supervisor","title":"restart_tolerance","content":"pub fn restart_tolerance(\n  builder: Builder,\n  intensity intensity: Int,\n  period period: Int,\n) -&gt; Builder\n To prevent a supervisor from getting into an infinite loop of child\n process terminations and restarts, a maximum restart intensity is\n defined using two integer values specified with keys intensity and\n period in the above map. Assuming the values MaxR for intensity and MaxT\n for period, then, if more than MaxR restarts occur within MaxT seconds,\n the supervisor terminates all child processes and then itself. The\n termination reason for the supervisor itself in that case will be\n shutdown. \n\n Intensity defaults to 1 and period defaults to 5.\nSynonyms:\neensy/otp/static_supervisor.restart_tolerance\neensy/otp/static_supervisor restart_tolerance","url":"eensy/otp/static_supervisor.html#restart_tolerance"},{"doc":"eensy/otp/static_supervisor","title":"significant","content":"pub fn significant(\n  child: ChildBuilder,\n  significant: Bool,\n) -&gt; ChildBuilder\n This defines if a child is considered significant for automatic\n self-shutdown of the supervisor.\n\n You most likely do not want to consider any children significant.\n\n This will be ignored if the supervisor auto shutdown is set to `Never`,\n which is the default.\n\n The default value for significance is `False`.\nSynonyms:\neensy/otp/static_supervisor.significant\neensy/otp/static_supervisor significant","url":"eensy/otp/static_supervisor.html#significant"},{"doc":"eensy/otp/static_supervisor","title":"start_link","content":"pub fn start_link(builder: Builder) -&gt; Result(Pid, Dynamic)\n\nSynonyms:\neensy/otp/static_supervisor.start_link\neensy/otp/static_supervisor start_link","url":"eensy/otp/static_supervisor.html#start_link"},{"doc":"eensy/otp/static_supervisor","title":"supervisor_child","content":"pub fn supervisor_child(\n  id id: String,\n  run starter: fn() -&gt; Result(Pid, a),\n) -&gt; ChildBuilder\n A special child that is a supervisor itself.\n\n id is used to identify the child specification internally by the\n supervisor.\n Notice that this identifier on occations has been called &quot;name&quot;. As far\n as possible, the terms &quot;identifier&quot; or &quot;id&quot; are now used but to keep\n backward compatibility, some occurences of &quot;name&quot; can still be found, for\n example in error messages.\n\nSynonyms:\neensy/otp/static_supervisor.supervisor_child\neensy/otp/static_supervisor supervisor_child","url":"eensy/otp/static_supervisor.html#supervisor_child"},{"doc":"eensy/otp/static_supervisor","title":"timeout","content":"pub fn timeout(child: ChildBuilder, ms ms: Int) -&gt; ChildBuilder\n This defines the amount of milliseconds a child has to shut down before\n being brutal killed by the supervisor.\n\n If not set the default for a child is 5000ms.\n\n This will be ignored if the child is a supervisor itself.\n\nSynonyms:\neensy/otp/static_supervisor.timeout\neensy/otp/static_supervisor timeout","url":"eensy/otp/static_supervisor.html#timeout"},{"doc":"eensy/otp/static_supervisor","title":"worker_child","content":"pub fn worker_child(\n  id id: String,\n  run starter: fn() -&gt; Result(Pid, a),\n) -&gt; ChildBuilder\n A regular child that is not also a supervisor.\n\n id is used to identify the child specification internally by the\n supervisor.\n Notice that this identifier on occations has been called &quot;name&quot;. As far\n as possible, the terms &quot;identifier&quot; or &quot;id&quot; are now used but to keep\n backward compatibility, some occurences of &quot;name&quot; can still be found, for\n example in error messages.\n\nSynonyms:\neensy/otp/static_supervisor.worker_child\neensy/otp/static_supervisor worker_child","url":"eensy/otp/static_supervisor.html#worker_child"},{"doc":"eensy/otp/static_supervisor","title":"eensy/otp/static_supervisor","content":" Bindings to Erlang/OTP&#39;s `supervisor` module.\n\n For further detail see the Erlang documentation:\n &lt;https://www.erlang.org/doc/apps/stdlib/supervisor.html&gt;.\n\n # Example\n\n ```gleam\n import gleam/erlang/process.{type Pid}\n import gleam/otp/static_supervisor as sup\n \n pub fn start_supervisor() {\n   sup.new(sup.OneForOne)\n   |&gt; sup.add(sup.worker_child(&quot;db&quot;, start_database_connection))\n   |&gt; sup.add(sup.worker_child(&quot;workers&quot;, start_workers))\n   |&gt; sup.add(sup.worker_child(&quot;web&quot;, start_http_server))\n   |&gt; sup.start_link\n }\n ```","url":"eensy/otp/static_supervisor.html"},{"doc":"eensy/otp/task","title":"AwaitError","content":"pub type AwaitError {\n  Timeout\n  Exit(reason: Dynamic)\n}\n\nTimeout\n\n\nExit(reason: Dynamic)\n\n\nSynonyms:\neensy/otp/task.AwaitError\neensy/otp/task AwaitError","url":"eensy/otp/task.html#AwaitError"},{"doc":"eensy/otp/task","title":"Task","content":"pub opaque type Task(value)\n\n\nSynonyms:\neensy/otp/task.Task\neensy/otp/task Task","url":"eensy/otp/task.html#Task"},{"doc":"eensy/otp/task","title":"async","content":"pub fn async(work: fn() -&gt; a) -&gt; Task(a)\n Spawn a task process that calls a given function in order to perform some\n work. The result of this function is send back to the parent and can be\n received using the `await` function.\n\n See the top level module documentation for more information on async/await.\n\nSynonyms:\neensy/otp/task.async\neensy/otp/task async","url":"eensy/otp/task.html#async"},{"doc":"eensy/otp/task","title":"await","content":"pub fn await(task: Task(a), timeout: Int) -&gt; a\n Wait for the value computed by a task.\n\n If the a value is not received before the timeout has elapsed or if the\n task process crashes then this function crashes.\n\nSynonyms:\neensy/otp/task.await\neensy/otp/task await","url":"eensy/otp/task.html#await"},{"doc":"eensy/otp/task","title":"try_await","content":"pub fn try_await(\n  task: Task(a),\n  timeout: Int,\n) -&gt; Result(a, AwaitError)\n Wait for the value computed by a task.\n\n If the a value is not received before the timeout has elapsed or if the\n task process crashes then an error is returned.\n\nSynonyms:\neensy/otp/task.try_await\neensy/otp/task try_await","url":"eensy/otp/task.html#try_await"},{"doc":"eensy/otp/task","title":"eensy/otp/task","content":" A task is a kind of process that performs a single task and then shuts\n down. Commonly tasks are used to convert sequential code into concurrent\n code by performing computation in another process.\n\n ```gleam\n let task = task.async(fn() { do_some_work() })\n let value = do_some_other_work()\n value + task.await(task, 100)\n ```\n\n Tasks spawned with async can be awaited on by their caller process (and\n only their caller) as shown in the example above. They are implemented by\n spawning a process that sends a message to the caller once the given\n computation is performed.\n\n There are two important things to consider when using `async`:\n\n 1. If you are using async tasks, you must await a reply as they are always\n    sent.\n\n 2. async tasks link the caller and the spawned process. This means that,\n    if the caller crashes, the task will crash too and vice-versa. This is\n    on purpose: if the process meant to receive the result no longer\n    exists, there is no purpose in completing the computation.\n\n This module is inspired by Elixir&#39;s [Task module][1].\n\n [1]: https://hexdocs.pm/elixir/master/Task.html\n","url":"eensy/otp/task.html"}]);